from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt

import views

urlpatterns = patterns(
    '',
    url(r'^$', views.PoolList.as_view(), name='pool-list'),
    url(r'^pool/(?P<pool>[\w-]+)/$', views.PoolInfo.as_view(), name='pool-info'),
    url(r'^pool/(?P<pool>[\w-]+)/rebalance/$', csrf_exempt(views.PoolRebalance.as_view()),
        name='pool-rebalance'),
    url(r'^node/(?P<address>[http://\w.:1-9-]+)/remove/$', views.NodeRemove.as_view(),
        name='node-remove'),
    url(r'^node/add/$', csrf_exempt(views.NodeAdd.as_view()), name='node-add'),
    url(r'^deploys/$', views.ListDeploy.as_view(), name='list-deploys'),
    url(r'^deploys/(?P<deploy>[\s\w@\.-]+)/$', views.DeployInfo.as_view(), name='deploy-info'),
    url(r'^healing/$', views.ListHealing.as_view(), name='list-healing'),
    url(r'^templates.json$', views.TemplateListJson.as_view(), name='template-list-json'),
    url(r'^(?P<address>[http://\w.:1-9-]+)/containers/$', views.NodeInfoJson.as_view(),
        name='node-info-json'),
    url(r'^(?P<address>[http://\w.:1-9-]+)/$', views.NodeInfo.as_view(),
        name='node-info'),
)



import copy

from django.test import TestCase
from django.test.client import RequestFactory
from django.http import Http404

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import DeployInfo

from mock import patch, Mock


class InfoViewTest(TestCase):

    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}
        self.data = {u'Id': u'53e143cb874ccb1f68000001',
                     u'App': u'g1',
                     u'Timestamp': u'18-08-2014 11:29:32',
                     u'Duration': u'00m23s',
                     u'Commit': u'e82nn93nd93mm12o2ueh83dhbd3iu112',
                     u'Error': u'',
                     u'Diff': u'test_diff'}

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view(self, token_is_valid, get):
        token_is_valid.return_value = True
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = copy.deepcopy(self.data)
        get.return_value = response_mock
        response = DeployInfo.as_view()(self.request,
                                        deploy="53e143cb874ccb1f68000001")
        self.assertIn("deploys/deploy_details.html", response.template_name)
        expected = copy.deepcopy(self.data)
        expected["Diff"] = u"<div class=\"highlight\"><pre>%s\n</pre></div>\n" % self.data["Diff"]
        self.assertDictEqual(expected, response.context_data['deploy'])
        get.assert_called_with(
            '{0}/deploys/{1}'.format(settings.TSURU_HOST,
                                     "53e143cb874ccb1f68000001"),
            headers={'authorization': 'admin'}
        )

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view_without_diff(self, token_is_valid, get):
        token_is_valid.return_value = True
        data = copy.deepcopy(self.data)
        del data["Diff"]
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = copy.deepcopy(data)
        get.return_value = response_mock
        response = DeployInfo.as_view()(self.request, deploy="53e143cb874ccb1f68000001")
        self.assertIn("deploys/deploy_details.html", response.template_name)
        expected = copy.deepcopy(data)
        expected["Diff"] = None
        self.assertDictEqual(expected, response.context_data["deploy"])
        get.assert_called_with(
            '{0}/deploys/{1}'.format(settings.TSURU_HOST, "53e143cb874ccb1f68000001"),
            headers={'authorization': 'admin'}
        )

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view_on_error(self, token_is_valid, get):
        token_is_valid.return_value = True
        response_mock = Mock(status_code=404)
        error_msg = "No JSON object could be decoded"
        response_mock.json.side_effect = ValueError(error_msg)
        get.return_value = response_mock

        with self.assertRaises(Http404):
            DeployInfo.as_view()(self.request, deploy="53e143cb874ccb1f68000001")



from django.http import HttpResponse, HttpResponseBadRequest

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from .backend import AppBackend, TsuruMetricsBackend, ComponentFilter, NodeMetricsBackend, NodesMetricsBackend
import json
import requests


class Metric(LoginRequiredView):
    def get(self, *args, **kwargs):
        token = self.request.session.get('tsuru_token')
        metric = self.request.GET.get("metric")
        if not metric:
            return HttpResponseBadRequest()

        interval = self.request.GET.get("interval")
        date_range = self.request.GET.get("date_range")
        target = kwargs['target']

        backend = self.get_metrics_backend(metric=metric, target=target, date_range=date_range, token=token)
        if backend is None:
            return HttpResponseBadRequest()

        data = getattr(backend, metric)(interval=interval)
        return HttpResponse(json.dumps(data))


class AppMetric(Metric):
    def get_app(self, app_name):
        url = '{}/apps/{}'.format(settings.TSURU_HOST, app_name)
        return requests.get(url, headers=self.authorization).json()

    def get_envs(self, request, app_name):
        url = '{}/apps/{}/env'.format(settings.TSURU_HOST, app_name)
        data = requests.get(url, headers=self.authorization).json()
        envs = {}

        for env in data:
            envs[env['name']] = env['value']

        return envs

    def get_metrics_backend(self, metric, target, date_range, token):
        process_name = self.request.GET.get("process_name")
        app = self.get_app(target)
        app["envs"] = self.get_envs(self.request, target)
        return AppBackend(app=app, token=token, date_range=date_range, process_name=process_name)


class ComponentMetric(Metric):
    def get_metrics_backend(self, metric, target, date_range, token):
        return TsuruMetricsBackend(filter=ComponentFilter(
            component=target, date_range=date_range), date_range=date_range)


class NodeMetric(Metric):
    def get_metrics_backend(self, metric, target, date_range, token):
        return NodeMetricsBackend(addr=target, date_range=date_range)


class PoolMetric(Metric):
    def get_pool_nodes(self, pool_name):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        pool_nodes = []
        if response.status_code != 204:
            nodes = response.json().get("nodes", [])

            for node in nodes:
                if node["Metadata"].get("pool", "") == pool_name:
                    addr = node["Address"].split("http://")[-1].split(":")[0]
                    pool_nodes.append(addr)

        return pool_nodes

    def get_metrics_backend(self, metric, target, date_range, token):
        addrs = self.get_pool_nodes(target)
        return NodesMetricsBackend(addrs=addrs, date_range=date_range)



from django.conf.urls import patterns, url

import views


urlpatterns = patterns(
    '',
    url(r'^$', views.ListComponent.as_view(), name='list-component'),
    url(r'^list.json$', views.ListComponentJson.as_view(), name='list-component-json')
)



import os


TSURU_HOST = os.environ.get("TSURU_HOST", "http://localhost:8080")
ELASTICSEARCH_HOST = os.environ.get("ELASTICSEARCH_HOST", "http://localhost:9200")
ELASTICSEARCH_INDEX = os.environ.get("ELASTICSEARCH_INDEX", "measure-tsuru")



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.apps.views import AppRollback


class AppRollbackViewTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.request = self.factory.get("/")
        self.request.session = {"tsuru_token": "admin"}

    def test_should_require_login_to_rollback_app(self):
        assert issubclass(AppRollback, LoginRequiredView)

    @patch("requests.post")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_post_with_valid_app_and_image(self, token_is_valid, post):
        token_is_valid.return_value = True
        m = Mock(status_code=200, content="")
        post.return_value = m
        response = AppRollback.as_view()(
            self.request, app_name='test', image='localhost:3030/tsuru/app-test:v44')
        self.assertEqual(302, response.status_code)

    @patch("requests.post")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_post_with_invalid_app_and_image(self, token_is_valid, post):
        token_is_valid.return_value = True
        m = Mock(status_code=500, content="")
        post.return_value = m
        response = AppRollback.as_view()(
            self.request, app_name='test', image='localhost:3030/tsuru/app-test:v44')
        self.assertEqual('NOT OK', response.content)



from django.conf.urls import patterns, url

import views


urlpatterns = patterns(
    '',
    url(r'^$', views.List.as_view(), name='team-list'),
    url(r'^(?P<team>[\s\w@\.-]+)/remove/$', views.Remove.as_view(),
        name='team-remove'),
    url(r'^add/$', views.Add.as_view(),
        name='team-add'),
    url(r'^(?P<team>[\s\w@\.-]+)/user/add/$', views.AddUser.as_view(),
        name='team-user-add'),
    url(r'^(?P<team>[\s\w@\.-]+)/user/(?P<user>.*)/remove/$',
        views.RemoveUser.as_view(), name='team-user-remove'),
    url(r'^(?P<team>[\s\w@\.-]+)/$', views.Info.as_view(), name='team-info'),
)



from dateutil import parser
from django import template

import time

register = template.Library()


@register.filter
def string_to_date(value):
    deploy_date = parser.parse(value)
    return deploy_date


@register.filter
def time_to_string(value):
    t = time.gmtime(float(value) / (1000 * 1000 * 1000))
    fmt = '%Mm%Ss'
    if t.tm_hour > 0:
        fmt = '%Hh{}'.format(fmt)
    return time.strftime(fmt, t)



from django.http import HttpResponseRedirect
from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from mock import Mock, patch

import httpretty

from tsuru_dashboard import settings
from tsuru_dashboard.auth.forms import LoginForm
from tsuru_dashboard.auth.views import Login


class LoginViewTest(TestCase):
    def setUp(self):
        httpretty.enable()
        url = "{}/users/info".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, url, status=200, body='{"Permissions": []}')

    @patch("requests.get")
    def test_login_get(self, get_mock):
        request = RequestFactory().get("/")
        request.session = {"next_url": "/"}
        response = Login.as_view()(request)
        self.assertIn("auth/login.html", response.template_name)
        form = response.context_data["form"]
        self.assertIsInstance(form, LoginForm)

    @patch("requests.get")
    def test_should_validate_data_from_post(self, get_mock):
        data = {"username": "invalid name", "password": ""}
        request = RequestFactory().post("/", data)
        request.session = {"next_url": "/"}
        response = Login.as_view()(request)
        form = response.context_data["form"]
        self.assertIn("auth/login.html", response.template_name)
        self.assertIsInstance(form, LoginForm)
        self.assertEqual("invalid name", form.data["username"])

    @patch("requests.post")
    @patch("requests.get")
    def test_should_return_invalid_message_when_user_doesnt_exist(self, get_mock, post):
        data = {"username": "invalid@email.com", "password": "123456"}
        post.return_value = Mock(status_code=500, content="user doesn't exist")
        response = self.client.post(reverse("login"), data, follow=True)
        error = response.context_data["form"].errors["__all__"]
        self.assertEqual(["user doesn't exist"], error)

    @patch("requests.post")
    @patch("requests.get")
    def test_should_send_request_post_to_tsuru_with_args_expected(self, get_mock, post):
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {"next_url": "/"}
        expected_url = "%s/users/valid@email.com/tokens" % settings.TSURU_HOST
        Login.as_view()(request)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(expected_url,
                                data='{"password": "123456"}')

    @patch("requests.post")
    def test_should_set_token_in_the_session(self, post):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        Login.as_view()(request)
        self.assertEqual("type my beautiful token", request.session["tsuru_token"])

    @patch("requests.post")
    def test_set_admin_permission_in_session(self, post):
        url = "{}/users/info".format(settings.TSURU_HOST)
        httpretty.register_uri(
            httpretty.GET, url, status=200, body='{"Permissions": [{"Name": "", "ContextType": "global"}]}'
        )
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        Login.as_view()(request)
        self.assertDictEqual({"healing": True, "admin": True}, request.session["permissions"])

    @patch("requests.post")
    def test_set_healing_permission_in_session(self, post):
        url = "{}/users/info".format(settings.TSURU_HOST)
        httpretty.register_uri(
            httpretty.GET, url, status=200, body='{"Permissions": [{"Name": "healing.read"}]}'
        )
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        Login.as_view()(request)
        self.assertDictEqual({"healing": True, "admin": False}, request.session["permissions"])

    @patch("requests.post")
    def test_should_set_username_in_the_session(self, post):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        Login.as_view()(request)
        self.assertEqual(data["username"], request.session["username"])

    @patch("requests.post")
    def test_redirect_to_team_creation_when_login_is_successful(self, post):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        response = Login.as_view()(request)
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual("/apps", response["Location"])

    @patch("requests.post")
    def test_redirect_to_team_creation_settings_false(self, post):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        response = Login.as_view()(request)
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual("/apps", response["Location"])

    @patch("requests.post")
    def test_redirect_to_apps(self, post):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"token": "my beautiful token"}
        post.return_value = response_mock
        data = {"username": "valid@email.com", "password": "123456"}
        request = RequestFactory().post("/", data)
        request.session = {}
        response = Login.as_view()(request)
        self.assertIsInstance(response, HttpResponseRedirect)
        self.assertEqual("/apps", response["Location"])

    @patch("requests.get")
    def test_scheme_info(self, get_mock):
        view = Login()
        expected_url = "{}/auth/scheme".format(settings.TSURU_HOST)

        self.assertDictEqual(view.scheme_info(), {})
        get_mock.assert_called_with(expected_url)

        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {"name": "oauth"}
        get_mock.return_value = response_mock

        self.assertDictEqual(view.scheme_info(), {"name": "oauth"})

        get_mock.assert_called_with(expected_url)

    @patch("requests.get")
    def test_get_context_data(self, get_mock):
        request = RequestFactory().get("/")
        request.session = {"next_url": "/"}
        request.META["HTTP_HOST"] = "localhost:3333"
        view = Login()
        view.request = request

        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {
            "name": "oauth",
            "data": {
                "authorizeUrl":
                "http://something.com/?redirect=__redirect_url__"
            }
        }
        get_mock.return_value = response_mock

        data = view.get_context_data()

        self.assertDictEqual(data["scheme_info"], {
            "name": "oauth",
            "data": {
                "authorizeUrl":
                "http://something.com/?redirect=__redirect_url__"
            },
        })
        self.assertEqual(
            data["authorize_url"],
            "http://something.com/?redirect=http://localhost:3333/auth/callback/"
        )

    @patch("requests.get")
    def test_get_context_data_with_data_is_none(self, get_mock):
        request = RequestFactory().get("/")
        request.META["HTTP_HOST"] = "localhost:3333"
        request.session = {"next_url": "/"}
        view = Login()
        view.request = request

        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {
            "name": "native",
            "data": None
        }
        get_mock.return_value = response_mock

        data = view.get_context_data()

        self.assertDictEqual(data["scheme_info"], {
            "name": "native",
            "data": None,
        })
